# Grun human pancreas (CEL-seq2) 

<script>
document.addEventListener("click", function (event) {
    if (event.target.classList.contains("rebook-collapse")) {
        event.target.classList.toggle("active");
        var content = event.target.nextElementSibling;
        if (content.style.display === "block") {
            content.style.display = "none";
        } else {
            content.style.display = "block";
        }
    }
})
</script>

<style>
.rebook-collapse {
  background-color: #eee;
  color: #444;
  cursor: pointer;
  padding: 18px;
  width: 100%;
  border: none;
  text-align: left;
  outline: none;
  font-size: 15px;
}

.rebook-content {
  padding: 0 18px;
  display: none;
  overflow: hidden;
  background-color: #f1f1f1;
}
</style>

## Introduction

This workflow performs an analysis of the @grun2016denovo CEL-seq2 dataset consisting of human pancreas cells from various donors.

## Data loading


```r
library(scRNAseq)
sce.grun <- GrunPancreasData()
```

We convert to Ensembl identifiers, and we remove duplicated genes or genes without Ensembl IDs.


```r
library(org.Hs.eg.db)
gene.ids <- mapIds(org.Hs.eg.db, keys=rowData(sce.grun)$symbol,
    keytype="SYMBOL", column="ENSEMBL")

keep <- !is.na(gene.ids) & !duplicated(gene.ids)
sce.grun <- sce.grun[keep,]
rownames(sce.grun) <- gene.ids[keep]
```

## Quality control


```r
unfiltered <- sce.grun
```

This dataset lacks mitochondrial genes so we will do without them for quality control.
We compute the median and MAD while blocking on the donor;
for donors where the assumption of a majority of high-quality cells seems to be violated (Figure \@ref(fig:unref-grun-qc-dist)),
we compute an appropriate threshold using the other donors as specified in the `subset=` argument.


```r
library(scater)
stats <- perCellQCMetrics(sce.grun)

qc <- quickPerCellQC(stats, percent_subsets="altexps_ERCC_percent",
    batch=sce.grun$donor,
    subset=sce.grun$donor %in% c("D17", "D7", "D2"))

sce.grun <- sce.grun[,!qc$discard]
```


```r
colData(unfiltered) <- cbind(colData(unfiltered), stats)
unfiltered$discard <- qc$discard

gridExtra::grid.arrange(
    plotColData(unfiltered, x="donor", y="sum", colour_by="discard") +
        scale_y_log10() + ggtitle("Total count"),
    plotColData(unfiltered, x="donor", y="detected", colour_by="discard") +
        scale_y_log10() + ggtitle("Detected features"),
    plotColData(unfiltered, x="donor", y="altexps_ERCC_percent",
        colour_by="discard") + ggtitle("ERCC percent"),
    ncol=2
)
```

<div class="figure">
<img src="grun-pancreas_files/figure-html/unref-grun-qc-dist-1.png" alt="Distribution of each QC metric across cells from each donor of the Grun pancreas dataset. Each point represents a cell and is colored according to whether that cell was discarded." width="672" />
<p class="caption">(\#fig:unref-grun-qc-dist)Distribution of each QC metric across cells from each donor of the Grun pancreas dataset. Each point represents a cell and is colored according to whether that cell was discarded.</p>
</div>


```r
colSums(as.matrix(qc), na.rm=TRUE)
```

```
##              low_lib_size            low_n_features high_altexps_ERCC_percent 
##                       450                       510                       606 
##                   discard 
##                       665
```

## Normalization 


```r
library(scran)
set.seed(1000) # for irlba. 
clusters <- quickCluster(sce.grun)
sce.grun <- computeSumFactors(sce.grun, clusters=clusters)
sce.grun <- logNormCounts(sce.grun)
```


```r
summary(sizeFactors(sce.grun))
```

```
##    Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
##   0.093   0.505   0.796   1.000   1.226  12.102
```


```r
plot(librarySizeFactors(sce.grun), sizeFactors(sce.grun), pch=16,
    xlab="Library size factors", ylab="Deconvolution factors", log="xy")
```

<div class="figure">
<img src="grun-pancreas_files/figure-html/unref-grun-norm-1.png" alt="Relationship between the library size factors and the deconvolution size factors in the Grun pancreas dataset." width="672" />
<p class="caption">(\#fig:unref-grun-norm)Relationship between the library size factors and the deconvolution size factors in the Grun pancreas dataset.</p>
</div>

## Variance modelling

We block on a combined plate and donor factor.


```r
block <- paste0(sce.grun$sample, "_", sce.grun$donor)
dec.grun <- modelGeneVarWithSpikes(sce.grun, spikes="ERCC", block=block)
top.grun <- getTopHVGs(dec.grun, prop=0.1)
```

We examine the number of cells in each level of the blocking factor.


```r
table(block)
```

```
## block
##                  CD13+ sorted cells_D17       CD24+ CD44+ live sorted cells_D17 
##                                      86                                      87 
##                  CD63+ sorted cells_D10                TGFBR3+ sorted cells_D17 
##                                      41                                      90 
## exocrine fraction, live sorted cells_D2 exocrine fraction, live sorted cells_D3 
##                                      82                                       7 
##        live sorted cells, library 1_D10        live sorted cells, library 1_D17 
##                                      33                                      88 
##         live sorted cells, library 1_D3         live sorted cells, library 1_D7 
##                                      24                                      85 
##        live sorted cells, library 2_D10        live sorted cells, library 2_D17 
##                                      35                                      83 
##         live sorted cells, library 2_D3         live sorted cells, library 2_D7 
##                                      27                                      84 
##         live sorted cells, library 3_D3         live sorted cells, library 3_D7 
##                                      16                                      83 
##         live sorted cells, library 4_D3         live sorted cells, library 4_D7 
##                                      29                                      83
```


```r
par(mfrow=c(6,3))
blocked.stats <- dec.grun$per.block
for (i in colnames(blocked.stats)) {
    current <- blocked.stats[[i]]
    plot(current$mean, current$total, main=i, pch=16, cex=0.5,
        xlab="Mean of log-expression", ylab="Variance of log-expression")
    curfit <- metadata(current)
    points(curfit$mean, curfit$var, col="red", pch=16)
    curve(curfit$trend(x), col='dodgerblue', add=TRUE, lwd=2)
}
```

<div class="figure">
<img src="grun-pancreas_files/figure-html/unref-416b-variance-1.png" alt="Per-gene variance as a function of the mean for the log-expression values in the Grun pancreas dataset. Each point represents a gene (black) with the mean-variance trend (blue) fitted to the spike-in transcripts (red) separately for each donor." width="672" />
<p class="caption">(\#fig:unref-416b-variance)Per-gene variance as a function of the mean for the log-expression values in the Grun pancreas dataset. Each point represents a gene (black) with the mean-variance trend (blue) fitted to the spike-in transcripts (red) separately for each donor.</p>
</div>

## Data integration


```r
library(batchelor)
set.seed(1001010)
merged.grun <- fastMNN(sce.grun, subset.row=top.grun, batch=sce.grun$donor)
```


```r
metadata(merged.grun)$merge.info$lost.var
```

```
##           D10      D17       D2      D3      D7
## [1,] 0.031117 0.030683 0.000000 0.00000 0.00000
## [2,] 0.006801 0.010960 0.037498 0.00000 0.00000
## [3,] 0.003700 0.005269 0.007379 0.05046 0.00000
## [4,] 0.012195 0.014662 0.013852 0.01283 0.05271
```

## Dimensionality reduction


```r
set.seed(100111)
merged.grun <- runTSNE(merged.grun, dimred="corrected")
```

## Clustering


```r
snn.gr <- buildSNNGraph(merged.grun, use.dimred="corrected")
colLabels(merged.grun) <- factor(igraph::cluster_walktrap(snn.gr)$membership)
```


```r
table(Cluster=colLabels(merged.grun), Donor=merged.grun$batch)
```

```
##        Donor
## Cluster D10 D17  D2  D3  D7
##       1  17  74   3   2  77
##       2   6  11   5   7   6
##       3  12 128   0   0  62
##       4  28 108  43  13 118
##       5  32  70  31  80  28
##       6   5  13   0   0  10
##       7   4  13   0   0   1
##       8   5  17   0   1  33
```


```r
gridExtra::grid.arrange(
    plotTSNE(merged.grun, colour_by="label"),
    plotTSNE(merged.grun, colour_by="batch"),
    ncol=2
)
```

<div class="figure">
<img src="grun-pancreas_files/figure-html/unref-grun-tsne-1.png" alt="Obligatory $t$-SNE plots of the Grun pancreas dataset. Each point represents a cell that is colored by cluster (left) or batch (right)." width="672" />
<p class="caption">(\#fig:unref-grun-tsne)Obligatory $t$-SNE plots of the Grun pancreas dataset. Each point represents a cell that is colored by cluster (left) or batch (right).</p>
</div>

## Session Info {-}

<button class="rebook-collapse">View session info</button>
<div class="rebook-content">
```
R version 4.1.1 (2021-08-10)
Platform: x86_64-pc-linux-gnu (64-bit)
Running under: Ubuntu 20.04.3 LTS

Matrix products: default
BLAS:   /home/biocbuild/bbs-3.14-bioc/R/lib/libRblas.so
LAPACK: /home/biocbuild/bbs-3.14-bioc/R/lib/libRlapack.so

locale:
 [1] LC_CTYPE=en_US.UTF-8       LC_NUMERIC=C              
 [3] LC_TIME=en_GB              LC_COLLATE=C              
 [5] LC_MONETARY=en_US.UTF-8    LC_MESSAGES=en_US.UTF-8   
 [7] LC_PAPER=en_US.UTF-8       LC_NAME=C                 
 [9] LC_ADDRESS=C               LC_TELEPHONE=C            
[11] LC_MEASUREMENT=en_US.UTF-8 LC_IDENTIFICATION=C       

attached base packages:
[1] stats4    stats     graphics  grDevices utils     datasets  methods  
[8] base     

other attached packages:
 [1] batchelor_1.10.0            scran_1.22.0               
 [3] scater_1.22.0               ggplot2_3.3.5              
 [5] scuttle_1.4.0               org.Hs.eg.db_3.14.0        
 [7] AnnotationDbi_1.56.0        scRNAseq_2.7.2             
 [9] SingleCellExperiment_1.16.0 SummarizedExperiment_1.24.0
[11] Biobase_2.54.0              GenomicRanges_1.46.0       
[13] GenomeInfoDb_1.30.0         IRanges_2.28.0             
[15] S4Vectors_0.32.0            BiocGenerics_0.40.0        
[17] MatrixGenerics_1.6.0        matrixStats_0.61.0         
[19] BiocStyle_2.22.0            rebook_1.4.0               

loaded via a namespace (and not attached):
  [1] AnnotationHub_3.2.0           BiocFileCache_2.2.0          
  [3] igraph_1.2.7                  lazyeval_0.2.2               
  [5] BiocParallel_1.28.0           digest_0.6.28                
  [7] ensembldb_2.18.0              htmltools_0.5.2              
  [9] viridis_0.6.2                 fansi_0.5.0                  
 [11] magrittr_2.0.1                memoise_2.0.0                
 [13] ScaledMatrix_1.2.0            cluster_2.1.2                
 [15] limma_3.50.0                  Biostrings_2.62.0            
 [17] prettyunits_1.1.1             colorspace_2.0-2             
 [19] blob_1.2.2                    rappdirs_0.3.3               
 [21] ggrepel_0.9.1                 xfun_0.27                    
 [23] dplyr_1.0.7                   crayon_1.4.1                 
 [25] RCurl_1.98-1.5                jsonlite_1.7.2               
 [27] graph_1.72.0                  glue_1.4.2                   
 [29] gtable_0.3.0                  zlibbioc_1.40.0              
 [31] XVector_0.34.0                DelayedArray_0.20.0          
 [33] BiocSingular_1.10.0           scales_1.1.1                 
 [35] edgeR_3.36.0                  DBI_1.1.1                    
 [37] Rcpp_1.0.7                    viridisLite_0.4.0            
 [39] xtable_1.8-4                  progress_1.2.2               
 [41] dqrng_0.3.0                   bit_4.0.4                    
 [43] rsvd_1.0.5                    ResidualMatrix_1.4.0         
 [45] metapod_1.2.0                 httr_1.4.2                   
 [47] dir.expiry_1.2.0              ellipsis_0.3.2               
 [49] pkgconfig_2.0.3               XML_3.99-0.8                 
 [51] farver_2.1.0                  CodeDepends_0.6.5            
 [53] sass_0.4.0                    dbplyr_2.1.1                 
 [55] locfit_1.5-9.4                utf8_1.2.2                   
 [57] labeling_0.4.2                tidyselect_1.1.1             
 [59] rlang_0.4.12                  later_1.3.0                  
 [61] munsell_0.5.0                 BiocVersion_3.14.0           
 [63] tools_4.1.1                   cachem_1.0.6                 
 [65] generics_0.1.1                RSQLite_2.2.8                
 [67] ExperimentHub_2.2.0           evaluate_0.14                
 [69] stringr_1.4.0                 fastmap_1.1.0                
 [71] yaml_2.2.1                    knitr_1.36                   
 [73] bit64_4.0.5                   purrr_0.3.4                  
 [75] KEGGREST_1.34.0               AnnotationFilter_1.18.0      
 [77] sparseMatrixStats_1.6.0       mime_0.12                    
 [79] xml2_1.3.2                    biomaRt_2.50.0               
 [81] compiler_4.1.1                beeswarm_0.4.0               
 [83] filelock_1.0.2                curl_4.3.2                   
 [85] png_0.1-7                     interactiveDisplayBase_1.32.0
 [87] statmod_1.4.36                tibble_3.1.5                 
 [89] bslib_0.3.1                   stringi_1.7.5                
 [91] highr_0.9                     GenomicFeatures_1.46.0       
 [93] bluster_1.4.0                 lattice_0.20-45              
 [95] ProtGenerics_1.26.0           Matrix_1.3-4                 
 [97] vctrs_0.3.8                   pillar_1.6.4                 
 [99] lifecycle_1.0.1               BiocManager_1.30.16          
[101] jquerylib_0.1.4               BiocNeighbors_1.12.0         
[103] cowplot_1.1.1                 bitops_1.0-7                 
[105] irlba_2.3.3                   httpuv_1.6.3                 
[107] rtracklayer_1.54.0            R6_2.5.1                     
[109] BiocIO_1.4.0                  bookdown_0.24                
[111] promises_1.2.0.1              gridExtra_2.3                
[113] vipor_0.4.5                   codetools_0.2-18             
[115] assertthat_0.2.1              rjson_0.2.20                 
[117] withr_2.4.2                   GenomicAlignments_1.30.0     
[119] Rsamtools_2.10.0              GenomeInfoDbData_1.2.7       
[121] parallel_4.1.1                hms_1.1.1                    
[123] grid_4.1.1                    beachmat_2.10.0              
[125] rmarkdown_2.11                DelayedMatrixStats_1.16.0    
[127] Rtsne_0.15                    shiny_1.7.1                  
[129] ggbeeswarm_0.6.0              restfulr_0.0.13              
```
</div>
